"""
pytest local configuration plug-in
"""

import gc
import os.path
import subprocess
import sys
from shutil import rmtree

import pytest
import warnings

THISDIR = os.path.dirname(__file__)


@pytest.fixture(autouse=True)
def error_on_ResourceWarning():
    """This fixture captures ResourceWarning's and reports an "error"
    describing the file handles left open.

    This is shown regardless of how successful the test was, if a test fails
    and leaves files open then those files will be reported.  Ideally, even
    those files should be closed properly after a test failure or exception.

    Since only Python 3 and PyPy3 have ResourceWarning's, this context will
    have no effect when running tests on Python 2 or PyPy.

    Because of autouse=True, this function will be automatically enabled for
    all test_* functions in this module.

    This code is primarily based on the examples found here:
    https://stackoverflow.com/questions/24717027/convert-python-3-resourcewarnings-into-exception
    """
    try:
        ResourceWarning
    except NameError:
        # Python 2, PyPy
        yield
        return

    # Python 3, PyPy3
    with warnings.catch_warnings(record=True) as caught:
        warnings.resetwarnings()  # clear all filters
        warnings.simplefilter('ignore')  # ignore all
        warnings.simplefilter('always', ResourceWarning)  # noqa: F821
        yield  # run tests in this context
        gc.collect()  # run garbage collection (for pypy3)
        if caught:
            pytest.fail('The following file descriptors were not closed properly:\n' +
                        '\n'.join((str(warning.message) for warning in caught)),
                        pytrace=False)


@pytest.fixture(scope='session')
def wheels_and_eggs():
    """Build wheels and eggs from test distributions."""
    test_distributions = "complex-dist", "simple.dist", "headers.dist", "unicode.dist"
    files = []
    pwd = os.path.abspath(os.curdir)
    for dist in test_distributions:
        os.chdir(os.path.join(THISDIR, dist))
        subprocess.check_call([sys.executable, 'setup.py', 'bdist_egg', 'bdist_wheel'])
        dist_path = os.path.join(os.curdir, 'dist')
        files.extend([os.path.abspath(os.path.join(dist_path, fname))
                      for fname in os.listdir(dist_path)
                      if os.path.splitext(fname)[1] in ('.whl', '.egg')])

    os.chdir(pwd)
    yield files

    for dist in test_distributions:
        for subdir in ('build', 'dist'):
            try:
                rmtree(os.path.join(THISDIR, dist, subdir))
            except OSError:
                pass


@pytest.fixture(scope='session')
def wheel_paths(wheels_and_eggs):
    return sorted(fname for fname in wheels_and_eggs if fname.endswith('.whl'))


@pytest.fixture(scope='session')
def egg_paths(wheels_and_eggs):
    return sorted(fname for fname in wheels_and_eggs if fname.endswith('.egg'))



from wheel.install import WheelFile
from wheel.pep425tags import get_supported

WHEELPAT = "%(name)s-%(ver)s-%(pyver)s-%(abi)s-%(arch)s.whl"


def make_wheel(name, ver, pyver, abi, arch):
    name = WHEELPAT % {'name': name, 'ver': ver, 'pyver': pyver, 'abi': abi, 'arch': arch}
    return WheelFile(name)


# This relies on the fact that generate_supported will always return the
# exact pyver, abi, and architecture for its first (best) match.
sup = get_supported()
pyver, abi, arch = sup[0]
genver = 'py' + pyver[2:]
majver = genver[:3]

COMBINATIONS = (
    ('bar', '0.9', 'py2.py3', 'none', 'any'),
    ('bar', '0.9', majver, 'none', 'any'),
    ('bar', '0.9', genver, 'none', 'any'),
    ('bar', '0.9', pyver, abi, arch),
    ('bar', '1.3.2', majver, 'none', 'any'),
    ('bar', '3.1', genver, 'none', 'any'),
    ('bar', '3.1', pyver, abi, arch),
    ('foo', '1.0', majver, 'none', 'any'),
    ('foo', '1.1', pyver, abi, arch),
    ('foo', '2.1', majver + '0', 'none', 'any'),
    # This will not be compatible for Python x.0. Beware when we hit Python
    # 4.0, and don't test with 3.0!!!
    ('foo', '2.1', majver + '1', 'none', 'any'),
    ('foo', '2.1', pyver, 'none', 'any'),
    ('foo', '2.1', pyver, abi, arch),
)

WHEELS = [make_wheel(*args) for args in COMBINATIONS]


def test_comparison():
    for i in range(len(WHEELS)-1):
        for j in range(i):
            assert WHEELS[j] < WHEELS[i]



from wheel import tool


def test_convert_egg(egg_paths, tmpdir):
    tool.convert(egg_paths, str(tmpdir), verbose=False)
    assert len(tmpdir.listdir()) == len(egg_paths)


def test_unpack(wheel_paths, tmpdir):
    """
    Make sure 'wheel unpack' works.
    This also verifies the integrity of our testing wheel files.
    """
    for wheel_path in wheel_paths:
        tool.unpack(wheel_path, str(tmpdir))


def test_keygen():
    def get_keyring():
        WheelKeys, keyring = tool.get_keyring()

        class WheelKeysTest(WheelKeys):
            def save(self):
                pass

        class keyringTest:
            @classmethod
            def get_keyring(cls):
                class keyringTest2:
                    pw = None

                    def set_password(self, a, b, c):
                        self.pw = c

                    def get_password(self, a, b):
                        return self.pw

                return keyringTest2()

        return WheelKeysTest, keyringTest

    tool.keygen(get_keyring=get_keyring)



import json

import pytest

from wheel.signatures import keys

wheel_json = """
{
  "verifiers": [
    {
      "scope": "+",
      "vk": "bp-bjK2fFgtA-8DhKKAAPm9-eAZcX_u03oBv2RlKOBc"
    },
    {
      "scope": "+",
      "vk": "KAHZBfyqFW3OcFDbLSG4nPCjXxUPy72phP9I4Rn9MAo"
    },
    {
      "scope": "+",
      "vk": "tmAYCrSfj8gtJ10v3VkvW7jOndKmQIYE12hgnFu3cvk"
    }
  ],
  "signers": [
    {
      "scope": "+",
      "vk": "tmAYCrSfj8gtJ10v3VkvW7jOndKmQIYE12hgnFu3cvk"
    },
    {
      "scope": "+",
      "vk": "KAHZBfyqFW3OcFDbLSG4nPCjXxUPy72phP9I4Rn9MAo"
    }
  ],
  "schema": 1
}
"""


@pytest.fixture
def wheel_keys(tmpdir, monkeypatch):
    def load(*args):
        return [config_path.dirname]

    def save(*args):
        return config_path.dirname

    config_path = tmpdir.join('config.json')
    config_path.write(b'')

    monkeypatch.setattr(keys, 'load_config_paths', load)
    monkeypatch.setattr(keys, 'save_config_path', save)

    wk = keys.WheelKeys()
    wk.CONFIG_NAME = config_path.basename
    return wk


def test_load_save(wheel_keys):
    wheel_keys.data = json.loads(wheel_json)

    wheel_keys.add_signer('+', '67890')
    wheel_keys.add_signer('scope', 'abcdefg')

    wheel_keys.trust('epocs', 'gfedcba')
    wheel_keys.trust('+', '12345')

    wheel_keys.save()

    del wheel_keys.data
    wheel_keys.load()

    signers = wheel_keys.signers('scope')
    assert signers[0] == ('scope', 'abcdefg'), wheel_keys.data['signers']
    assert signers[1][0] == '+', wheel_keys.data['signers']

    trusted = wheel_keys.trusted('epocs')
    assert trusted[0] == ('epocs', 'gfedcba')
    assert trusted[1][0] == '+'

    wheel_keys.untrust('epocs', 'gfedcba')
    trusted = wheel_keys.trusted('epocs')
    assert ('epocs', 'gfedcba') not in trusted


def test_load_save_incomplete(wheel_keys):
    wheel_keys.data = json.loads(wheel_json)
    del wheel_keys.data['signers']
    wheel_keys.data['schema'] = wheel_keys.SCHEMA+1
    wheel_keys.save()
    pytest.raises(ValueError, wheel_keys.load)

    del wheel_keys.data['schema']
    wheel_keys.save()
    wheel_keys.load()



# coding: utf-8
from zipfile import ZipFile


def test_no_scripts(wheel_paths):
    """Make sure entry point scripts are not generated."""
    path = next(path for path in wheel_paths if 'complex-dist' in path)
    for entry in ZipFile(path).infolist():
        assert '.data/scripts/' not in entry.filename


def test_unicode_record(wheel_paths):
    path = next(path for path in wheel_paths if 'unicode.dist' in path)
    with ZipFile(path) as zf:
        record = zf.read('unicode.dist-0.1.dist-info/RECORD')

    assert u'åäö_日本語.py'.encode('utf-8') in record



from wheel.metadata import generate_requirements


def test_generate_requirements():
    extras_require = {
        'test': ['ipykernel', 'ipython', 'mock'],
        'test:python_version == "3.3"': ['pytest<3.3.0'],
        'test:python_version >= "3.4" or python_version == "2.7"': ['pytest'],
    }
    expected_metadata = [
        ('Provides-Extra',
         'test'),
        ('Requires-Dist',
         "ipykernel; extra == 'test'"),
        ('Requires-Dist',
         "ipython; extra == 'test'"),
        ('Requires-Dist',
         "mock; extra == 'test'"),
        ('Requires-Dist',
         'pytest (<3.3.0); (python_version == "3.3") and extra == \'test\''),
        ('Requires-Dist',
         'pytest; (python_version >= "3.4" or python_version == "2.7") and extra == \'test\''),
    ]
    generated_metadata = sorted(set(generate_requirements(extras_require)))
    assert generated_metadata == expected_metadata



import hashlib
import zipfile
from io import BytesIO

import pytest

import wheel.archive
import wheel.install


def test_verifying_zipfile():
    if not hasattr(zipfile.ZipExtFile, '_update_crc'):
        pytest.skip('No ZIP verification. Missing ZipExtFile._update_crc.')

    bio = BytesIO()
    zf = zipfile.ZipFile(bio, 'w')
    zf.writestr("one", b"first file")
    zf.writestr("two", b"second file")
    zf.writestr("three", b"third file")
    zf.close()

    # In default mode, VerifyingZipFile checks the hash of any read file
    # mentioned with set_expected_hash(). Files not mentioned with
    # set_expected_hash() are not checked.
    vzf = wheel.install.VerifyingZipFile(bio, 'r')
    vzf.set_expected_hash("one", hashlib.sha256(b"first file").digest())
    vzf.set_expected_hash("three", "blurble")
    vzf.open("one").read()
    vzf.open("two").read()
    pytest.raises(wheel.install.BadWheelFile, vzf.open("three").read)

    # In strict mode, VerifyingZipFile requires every read file to be
    # mentioned with set_expected_hash().
    vzf.strict = True
    pytest.raises(wheel.install.BadWheelFile, vzf.open, "two")

    vzf.set_expected_hash("two", None)
    vzf.open("two").read()


def test_pop_zipfile():
    bio = BytesIO()
    zf = wheel.install.VerifyingZipFile(bio, 'w')
    zf.writestr("one", b"first file")
    zf.writestr("two", b"second file")
    zf.close()

    pytest.raises(RuntimeError, zf.pop)

    zf = wheel.install.VerifyingZipFile(bio, 'a')
    zf.pop()
    zf.close()

    zf = wheel.install.VerifyingZipFile(bio, 'r')
    assert len(zf.infolist()) == 1


def test_zipfile_timestamp(tmpdir, monkeypatch):
    # An environment variable can be used to influence the timestamp on
    # TarInfo objects inside the zip.  See issue #143.
    for filename in ('one', 'two', 'three'):
        tmpdir.join(filename).write(filename + '\n')

    # The earliest date representable in TarInfos, 1980-01-01
    monkeypatch.setenv('SOURCE_DATE_EPOCH', '315576060')

    zip_base_name = str(tmpdir.join('dummy'))
    zip_filename = wheel.archive.make_wheelfile_inner(zip_base_name, str(tmpdir))
    with zipfile.ZipFile(zip_filename, 'r') as zf:
        for info in zf.infolist():
            assert info.date_time[:3] == (1980, 1, 1)


def test_zipfile_attributes(tmpdir):
    # With the change from ZipFile.write() to .writestr(), we need to manually
    # set member attributes.
    files = (('foo', 0o644), ('bar', 0o755))
    for filename, mode in files:
        path = tmpdir.join(filename)
        path.write(filename + '\n')
        path.chmod(mode)

    zip_base_name = str(tmpdir.join('dummy'))
    zip_filename = wheel.archive.make_wheelfile_inner(zip_base_name, str(tmpdir))
    with zipfile.ZipFile(zip_filename, 'r') as zf:
        for filename, mode in files:
            info = zf.getinfo(str(tmpdir.join(filename)))
            assert info.external_attr == (mode | 0o100000) << 16
            assert info.compress_type == zipfile.ZIP_DEFLATED



# Test wheel.
# The file has the following contents:
#   hello.pyd
#   hello/hello.py
#   hello/__init__.py
#   test-1.0.data/data/hello.dat
#   test-1.0.data/headers/hello.dat
#   test-1.0.data/scripts/hello.sh
#   test-1.0.dist-info/WHEEL
#   test-1.0.dist-info/METADATA
#   test-1.0.dist-info/RECORD
# The root is PLATLIB
# So, some in PLATLIB, and one in each of DATA, HEADERS and SCRIPTS.

import os
import shutil
from tempfile import mkdtemp

import wheel.pep425tags
import wheel.tool
from wheel.install import WheelFile

THISDIR = os.path.dirname(__file__)
TESTWHEEL = os.path.join(THISDIR, 'test-1.0-py2.py3-none-win32.whl')


def test_compatibility_tags():
    """Test compatibilty tags are working."""
    wf = WheelFile("package-1.0.0-cp32.cp33-noabi-noarch.whl")
    assert (list(wf.compatibility_tags) ==
            [('cp32', 'noabi', 'noarch'), ('cp33', 'noabi', 'noarch')])
    assert wf.arity == 2

    wf2 = WheelFile("package-1.0.0-1st-cp33-noabi-noarch.whl")
    wf2_info = wf2.parsed_filename.groupdict()
    assert wf2_info['build'] == '1st', wf2_info


def test_pick_best():
    """Test the wheel ranking algorithm."""
    def get_tags(res):
        info = res[-1].parsed_filename.groupdict()
        return info['pyver'], info['abi'], info['plat']

    cand_tags = [('py27', 'noabi', 'noarch'), ('py26', 'noabi', 'noarch'),
                 ('cp27', 'noabi', 'linux_i686'),
                 ('cp26', 'noabi', 'linux_i686'),
                 ('cp27', 'noabi', 'linux_x86_64'),
                 ('cp26', 'noabi', 'linux_x86_64')]
    cand_wheels = [WheelFile('testpkg-1.0-%s-%s-%s.whl' % t)
                   for t in cand_tags]

    supported = [('cp27', 'noabi', 'linux_i686'), ('py27', 'noabi', 'noarch')]
    supported2 = [('cp27', 'noabi', 'linux_i686'), ('py27', 'noabi', 'noarch'),
                  ('cp26', 'noabi', 'linux_i686'), ('py26', 'noabi', 'noarch')]
    supported3 = [('cp26', 'noabi', 'linux_i686'), ('py26', 'noabi', 'noarch'),
                  ('cp27', 'noabi', 'linux_i686'), ('py27', 'noabi', 'noarch')]

    for supp in (supported, supported2, supported3):
        def context():
            return list(supp)

        for wheel_ in cand_wheels:
            wheel_.context = context
        best = max(cand_wheels)
        assert list(best.tags)[0] == supp[0]

        # assert_equal(
        #     list(map(get_tags, pick_best(cand_wheels, supp, top=False))), supp)


def test_install():
    def check(*path):
        return os.path.exists(os.path.join(*path))

    def get_supported():
        return list(wheel.pep425tags.get_supported()) + [('py3', 'none', 'win32')]

    tempdir = mkdtemp()
    whl = WheelFile(TESTWHEEL, context=get_supported)
    assert whl.supports_current_python(get_supported)
    try:
        locs = {}
        for key in ('purelib', 'platlib', 'scripts', 'headers', 'data'):
            locs[key] = os.path.join(tempdir, key)
            os.mkdir(locs[key])
        whl.install(overrides=locs)
        assert len(os.listdir(locs['purelib'])) == 0
        assert check(locs['platlib'], 'hello.pyd')
        assert check(locs['platlib'], 'hello', 'hello.py')
        assert check(locs['platlib'], 'hello', '__init__.py')
        assert check(locs['data'], 'hello.dat')
        assert check(locs['headers'], 'hello.dat')
        assert check(locs['scripts'], 'hello.sh')
        assert check(locs['platlib'], 'test-1.0.dist-info', 'RECORD')
    finally:
        shutil.rmtree(tempdir)


def test_install_tool():
    """Slightly improve coverage of wheel.install"""
    wheel.tool.install([TESTWHEEL], force=True, dry_run=True)


def test_wheelfile_re():
    # Regression test for #208
    wf = WheelFile('foo-2-py3-none-any.whl')
    assert wf.distinfo_name == 'foo-2.dist-info'



# coding: utf-8
import os.path
import io
import re

from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
README = io.open(os.path.join(here, 'README.rst'), encoding='utf8').read()
CHANGES = io.open(os.path.join(here, 'CHANGES.txt'), encoding='utf8').read()

with io.open(os.path.join(here, 'wheel', '__init__.py'), encoding='utf8') as version_file:
    metadata = dict(re.findall(r"""__([a-z]+)__ = "([^"]+)""", version_file.read()))

setup(name='wheel',
      version=metadata['version'],
      description='A built-package format for Python.',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.4",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6"
      ],
      author='Daniel Holth',
      author_email='dholth@fastmail.fm',
      maintainer=u'Alex Grönholm',
      maintainer_email='alex.gronholm@nextday.fi',
      url='https://github.com/pypa/wheel',
      keywords=['wheel', 'packaging'],
      license='MIT',
      packages=find_packages(),
      python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
      extras_require={
          'signatures': ['keyring', 'keyrings.alt'],
          'signatures:sys_platform!="win32"': ['pyxdg'],
          'faster-signatures': ['ed25519ll'],
          'test': ['pytest >= 3.0.0', 'pytest-cov']
          },
      include_package_data=True,
      zip_safe=False,
      entry_points={
          'console_scripts': [
              'wheel=wheel.tool:main'
              ],
          'distutils.commands': [
              'bdist_wheel=wheel.bdist_wheel:bdist_wheel'
              ]
          }
      )



import os.path

from wheel import egg2wheel


def test_egg_re():
    """Make sure egg_info_re matches."""
    egg_names_path = os.path.join(os.path.dirname(__file__), 'eggnames.txt')
    with open(egg_names_path) as egg_names:
        for line in egg_names:
            line = line.strip()
            if line:
                assert egg2wheel.egg_info_re.match(line), line
